﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;


namespace HeuristicLab.Algorithms.ALPS.SteadyState {
  [Item("NamedSubScopeProcessor", "")]
  [StorableClass]
  public class NamedSubScopeProcessor : SingleSuccessorOperator {

    #region Parameter Properties
    private OperatorParameter OperatorParameter {
      get { return (OperatorParameter)Parameters["Operator"]; }
    }
    public ILookupParameter<IScope> TargetScopeParameter {
      get { return (ILookupParameter<IScope>)Parameters["TargetScope"]; }
    }
    #endregion

    public IOperator Operator {
      get { return OperatorParameter.Value; }
      set { OperatorParameter.Value = value; }
    }

    [StorableConstructor]
    protected NamedSubScopeProcessor(bool deserializing) : base(deserializing) { }
    public NamedSubScopeProcessor()
      : base() {
      Parameters.Add(new OperatorParameter("Operator", "The operator which should be applied on all sub-scopes of the current scope."));
      Parameters.Add(new LookupParameter<IScope>("TargetScope"));
    }
    protected NamedSubScopeProcessor(NamedSubScopeProcessor original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NamedSubScopeProcessor(this, cloner);
    }

    public override IOperation Apply() {
      var next = new OperationCollection(base.Apply());
      var targetScope = TargetScopeParameter.ActualValue;
      next.Insert(0, ExecutionContext.CreateOperation(Operator, targetScope));
      return next;
    }
  }
}